# -*- coding: utf-8 -*-
"""
Test Russian language parsing.
"""

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
# pylint:disable=redefined-builtin,wildcard-import,unused-wildcard-import
from builtins import *
# pylint:enable=redefined-builtin,wildcard-import,unused-wildcard-import
from future import standard_library
standard_library.install_aliases()

from nose.tools import assert_equal
from aloe.parser import Feature

SCENARIO = u"""
Сценарий: Сохранение базы курсов универитета в текстовый файл
    Допустим имеем в базе университет следующие курсы:
       | Название                | Длительность  |
       | Матан                   | 2 года        |
       | Основы программирования | 1 год         |
    Когда я сохраняю базу курсову в файл 'курсы.txt'
    Тогда в первой строке файла 'курсы.txt' строку 'Матан:2'
    И во второй строке файла 'курсы.txt' строку 'Основы программирования:1'
"""

SCENARIO_OUTLINE1 = u'''
Структура сценария: Заполнение пользователей в базу
    Допустим я заполняю в поле "имя" "<имя>"
    И я заполняю в поле "возраст"  "<возраст>"
    Если я сохраняю форму
    То я вижу сообщени "Студент <имя>, возраст <возраст>, успешно занесен в базу!"

Примеры:
    | имя  | возраст |
    | Вася | 22      |
    | Петя | 30      |
'''  # noqa

FEATURE = u'''
Функционал: Деление чисел
  Поскольку деление сложный процесс и люди часто допускают ошибки
  Нужно дать им возможность делить на калькуляторе

  Сценарий: Целочисленное деление
    Допустим я беру калькулятор
    Тогда я делю делимое на делитель и получаю частное
    | делимое | делитель | частное |
    | 100     | 2        | 50      |
    | 28      | 7        | 4       |
    | 0       | 5        | 0       |
'''


def parse_scenario(string, language=None):
    """Parse a scenario, prefixing it with a feature header."""
    feature = u"""
    Функция: parse_scenario
    """
    feature += string
    feature = Feature.from_string(feature, language=language)

    return feature.scenarios[0]


def test_scenario_ru_from_string():
    """
    Language: RU -> Scenario.from_string
    """

    scenario = parse_scenario(SCENARIO, language='ru')

    assert_equal(
        scenario.name,
        u'Сохранение базы курсов универитета в текстовый файл'
    )
    assert_equal(
        scenario.steps[0].hashes,
        (
            {u'Название': u'Матан',
             u'Длительность': u'2 года'},
            {u'Название': u'Основы программирования',
             u'Длительность': u'1 год'},
        )
    )


def test_scenario_outline1_ru_from_string():
    """
    Language: RU -> Scenario.from_string, with scenario outline, first case
    """

    scenario = parse_scenario(SCENARIO_OUTLINE1, language='ru')

    assert_equal(
        scenario.name,
        u'Заполнение пользователей в базу'
    )
    assert_equal(
        scenario.outlines,
        (
            {u'имя': u'Вася', u'возраст': '22'},
            {u'имя': u'Петя', u'возраст': '30'},
        )
    )


def test_feature_ru_from_string():
    """
    Language: RU -> Feature.from_string
    """

    feature = Feature.from_string(FEATURE, language='ru')

    assert_equal(
        feature.name,
        u'Деление чисел'
    )

    assert_equal(
        feature.description,
        u"Поскольку деление сложный процесс и люди часто допускают ошибки\n"
        u"Нужно дать им возможность делить на калькуляторе"
    )

    (scenario, ) = feature.scenarios

    assert_equal(
        scenario.name,
        u'Целочисленное деление'
    )

    assert_equal(
        scenario.steps[-1].hashes,
        (
            {u'делимое': '100', u'делитель': '2', u'частное': '50'},
            {u'делимое': '28', u'делитель': '7', u'частное': '4'},
            {u'делимое': '0', u'делитель': '5', u'частное': '0'},
        )
    )



"""
Test filesystem-related functions.
"""

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from future import standard_library
standard_library.install_aliases()

import unittest

from aloe.fs import path_to_module_name


class PathToModuleNameTest(unittest.TestCase):
    """
    Test path_to_module_name.
    """

    def test_path_to_module_name(self):
        """Test path_to_module_name."""

        self.assertEqual(
            'one.two.three',
            path_to_module_name('one/two/three.py')
        )

        self.assertEqual(
            'one.two',
            path_to_module_name('one/two/__init__.py')
        )



"""
Setup script.
"""

__version__ = '0.0.25'

from setuptools import setup, find_packages

if __name__ == '__main__':
    with \
            open('requirements.txt') as requirements, \
            open('test_requirements.txt') as test_requirements, \
            open('README.md') as readme:
        setup(
            name='aloe',
            version=__version__,
            description='Gherkin runner compatible with Lettuce',
            author='Alexey Kotlyarov',
            author_email='a@koterpillar.com',
            url='https://github.com/koterpillar/aloe',
            long_description=readme.read(),
            classifiers=[
                'License :: OSI Approved :: ' +
                'GNU General Public License v3 or later (GPLv3+)',
            ],

            packages=find_packages(exclude=['tests']),
            include_package_data=True,

            entry_points={
                'console_scripts': [
                    'aloe = aloe:main',
                ],
            },

            install_requires=requirements.readlines(),

            test_suite='tests',
            tests_require=test_requirements.readlines(),
        )



"""
Python tests for a test application.
"""

import os
import unittest


class NormalTestCase(unittest.TestCase):
    """
    Normal test which should not be picked up by the Gherkin runner.
    """

    @classmethod
    def setUpClass(cls):
        """
        Prevent the test from running in the main (Aloe) test suite.
        """

        if 'simple_app' not in os.getcwd():
            raise unittest.SkipTest()

    def test_failing_test(self):
        """
        A test to check whether the Python tests are running.
        """

        raise AssertionError("This test should not be run.")



"""
Test step loading.
"""

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from future import standard_library
standard_library.install_aliases()

import os
import unittest

from aloe.testing import (
    FeatureTest,
    in_directory,
)


@in_directory('tests/unit')
class FeatureTestTest(FeatureTest):
    """
    Test running features.
    """

    def test_run_good_feature_string(self):
        """
        Test running strings as features.
        """

        result = self.run_feature_string(
            """
            Feature: Empty feature

            Scenario: Empty scenario
                Given I do nothing
                Then I do nothing
            """
        )

        self.assertTrue(result.success)

    def test_run_feature_string_fail(self):
        """
        Test running a failing feature string.
        """

        result = self.run_feature_string(
            """
            Feature: Empty feature

            Scenario: Empty scenario
                Given I do nothing
                Then I fail
            """
        )

        self.assertFalse(result.success)

    def test_run_feature_string_parse_error(self):
        """
        Test running a misformatted feature string.
        """

        result = self.run_feature_string(
            """
            Not a feature
            """
        )

        self.assertFalse(result.success)


def relative(directory):
    """
    A directory relative to the one containing this file.
    """

    return os.path.join(os.path.dirname(__file__), directory)


@in_directory(relative('../../tests/functional'))
class InDirectoryTest(unittest.TestCase):
    """
    Test in_directory.
    """

    def test_in_directory_on_class(self):
        """
        Test in_directory on the containing class.
        """

        self.assertTrue(os.getcwd().endswith('tests/functional'))

    @in_directory(relative('../../tests'))
    def test_in_directory_on_method(self):
        """
        Test in_directory on the method.
        """

        self.assertTrue(os.getcwd().endswith('tests'))



"""
The Gherkin Domain for Sphinx_ provides additional directives for
documenting steps using Sphinx.

.. rst:directive:: .. gherkin:restep:: Sentence regex

    Provide the documentation for a Gherkin regular expression step.

    For example:

    .. code-block:: rst

        .. gherkin:restep:: (?:Given|When|And) I visit the supermarket

            I am at the supermarket.

    Is rendered as:

        .. gherkin:restep:: (?:Given|When|And) I visit the supermarket

            I am at the supermarket.

.. _Sphinx: http://sphinx-doc.org/
"""

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
# pylint:disable=redefined-builtin, unused-wildcard-import, wildcard-import
from builtins import *
# pylint:enable=redefined-builtin, unused-wildcard-import, wildcard-import
from future import standard_library
standard_library.install_aliases()

from sphinx.directives import ObjectDescription
from sphinx.domains import Domain
from sphinx import addnodes
from docutils import nodes


class GherkinREStep(ObjectDescription):
    """Description of a Gherkin Regex Step"""

    type_ = 'Step'

    def handle_signature(self, sig, signode):
        signode += addnodes.desc_annotation(self.type_ + ' ', self.type_ + ' ')
        signode += nodes.strong(sig, sig)

        return sig


class GherkinDomain(Domain):
    """Domain for Gherkin syntax"""

    name = 'gherkin'
    label = 'Gherkin'

    directives = {
        'restep': GherkinREStep,
    }

    def merge_domaindata(self, docnames, otherdata):
        raise NotImplementedError("FIXME")

    # pylint:disable=too-many-arguments
    def resolve_any_xref(self, env, fromdocname, builder, target,
                         node, contnode):
        """FIXME: implement"""
        return []
    # pylint:enable=too-many-arguments


def setup(app):
    """Initialize extension"""
    app.add_domain(GherkinDomain)



# -*- coding: utf-8 -*-
"""
Filesystem-related utilities.
"""

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from future import standard_library
standard_library.install_aliases()

import os
import fnmatch

from importlib import import_module
try:
    reload
except NameError:
    # pylint:disable=no-name-in-module,redefined-builtin
    from importlib import reload
    # pylint:enable=no-name-in-module,redefined-builtin


def path_to_module_name(filename):
    """Convert a path to a file to a Python module name."""

    filename = os.path.relpath(filename)

    dotted_path = []
    while True:
        filename, component = os.path.split(filename)
        dotted_path.insert(0, component)
        if filename == '':
            break

    dotted_path[-1] = os.path.splitext(dotted_path[-1])[0]
    if dotted_path[-1] == '__init__':
        dotted_path.pop()

    return '.'.join(dotted_path)


class FeatureLoader(object):
    """Loader class responsible for findind features and step
    definitions along a given path on filesystem"""

    @classmethod
    def find_and_load_step_definitions(cls, dir_):
        """
        Load the steps from the specified directory.
        """

        for path, _, files in os.walk(dir_):
            for filename in fnmatch.filter(files, '*.py'):
                # Import the module using its fully qualified name
                filename = os.path.relpath(os.path.join(path, filename))
                module_name = path_to_module_name(filename)

                module = import_module(module_name)
                reload(module)  # Make sure steps and hooks are registered

    @classmethod
    def find_feature_directories(cls, dir_):
        """
        Locate directories to load features from.

        The directories must be in a package directory (have '__init__.py' all
        the way up) and be named 'features'.
        """

        # A set of package directories discovered
        packages = set(dir_)

        for path, dirs, files in os.walk(dir_):
            # Is this a package?
            if '__init__.py' in files:
                packages.add(path)

            if path in packages:
                # Does this package have a feature directory?
                if 'features' in dirs:
                    yield os.path.join(path, 'features')

            else:
                # This is not a package, prune search
                dirs[:] = []
